
/*
 * This file has been auto-generated from the introspection data available
 * in the at-spi2-core repository. The D-Bus protocol is defined in this
 * repository, which can be found at:
 *
 * https://gitlab.gnome.org/GNOME/at-spi2-core
 *
 * DO NOT EDIT.
 */


const char *spi_org_a11y_atspi_Accessible =
"<interface name=\"org.a11y.atspi.Accessible\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <property name=\"Name\" type=\"s\" access=\"read\" />"
""
"    "
"    <property name=\"Description\" type=\"s\" access=\"read\" />"
""
"    "
"    <property name=\"Parent\" type=\"(so)\" access=\"read\">"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName\" value=\"QSpiObjectReference\" />"
"    </property>"
""
"    "
"    <property name=\"ChildCount\" type=\"i\" access=\"read\" />"
""
"    "
"    <property name=\"Locale\" type=\"s\" access=\"read\" />"
""
"    "
"    <property name=\"AccessibleId\" type=\"s\" access=\"read\" />"
""
"    "
"    <property name=\"HelpText\" type=\"s\" access=\"read\" />"
""
"    "
"    <method name=\"GetChildAtIndex\">"
"      <arg direction=\"in\" name=\"index\" type=\"i\" />"
"      <arg direction=\"out\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiObjectReference\" />"
"    </method>"
""
"    "
"    <method name=\"GetChildren\">"
"      <arg direction=\"out\" type=\"a(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiObjectReferenceArray\" />"
"    </method>"
""
"    "
"    <method name=\"GetIndexInParent\">"
"      <arg direction=\"out\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetRelationSet\">"
"      <arg direction=\"out\" type=\"a(ua(so))\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiRelationArray\" />"
"    </method>"
""
"    "
"    <method name=\"GetRole\">"
"      <arg direction=\"out\" type=\"u\" />"
"    </method>"
""
"    "
"    <method name=\"GetRoleName\">"
"      <arg direction=\"out\" type=\"s\" />"
"    </method>"
""
"    "
"    <method name=\"GetLocalizedRoleName\">"
"      <arg direction=\"out\" type=\"s\" />"
"    </method>"
""
"    "
"    <method name=\"GetState\">"
"      <arg direction=\"out\" type=\"au\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiIntList\" />"
"    </method>"
""
"    "
"    <method name=\"GetAttributes\">"
"      <arg direction=\"out\" type=\"a{ss}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiAttributeSet\" />"
"    </method>"
""
"    "
"    <method name=\"GetApplication\">"
"      <arg direction=\"out\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiObjectReference\" />"
"    </method>"
""
"    "
"    <method name=\"GetInterfaces\">"
"      <arg direction=\"out\" type=\"as\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_Action =
"<interface name=\"org.a11y.atspi.Action\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <property name=\"NActions\" type=\"i\" access=\"read\" />"
""
"    "
"    <method name=\"GetDescription\">"
"      <arg type=\"i\" name=\"index\" direction=\"in\" />"
"      <arg type=\"s\" direction=\"out\" />"
"    </method>"
""
"    "
"    <method name=\"GetName\">"
"      <arg type=\"i\" name=\"index\" direction=\"in\" />"
"      <arg type=\"s\" direction=\"out\" />"
"    </method>"
""
"    "
"    <method name=\"GetLocalizedName\">"
"      <arg type=\"i\" name=\"index\" direction=\"in\" />"
"      <arg type=\"s\" direction=\"out\" />"
"    </method>"
""
"    "
"    <method name=\"GetKeyBinding\">"
"      <arg type=\"i\" name=\"index\" direction=\"in\" />"
"      <arg type=\"s\" direction=\"out\" />"
"    </method>"
""
"    "
"    <method name=\"GetActions\">"
"      <arg direction=\"out\" type=\"a(sss)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiActionArray\" />"
"    </method>"
""
"    "
"    <method name=\"DoAction\">"
"      <arg direction=\"in\" name=\"index\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_Application =
"<interface name=\"org.a11y.atspi.Application\">"
""
"    "
"    <property name=\"ToolkitName\" type=\"s\" access=\"read\" />"
""
"    "
"    <property name=\"Version\" type=\"s\" access=\"read\">"
"      <annotation name=\"org.freedesktop.DBus.Deprecated\" value=\"true\" />"
"    </property>"
""
"    "
"    <property name=\"ToolkitVersion\" type=\"s\" access=\"read\" />"
""
"    "
"    <property name=\"AtspiVersion\" type=\"s\" access=\"read\" />"
""
"    "
"    <property name=\"InterfaceVersion\" type=\"u\" access=\"read\" />"
""
"    "
"    <property name=\"Id\" type=\"i\" access=\"readwrite\" />"
""
"    "
"    <method name=\"GetLocale\">"
"      <arg direction=\"in\" name=\"lctype\" type=\"u\" />"
"      <arg direction=\"out\" type=\"s\" />"
"    </method>"
""
"    "
"    <method name=\"GetApplicationBusAddress\">"
"      <arg direction=\"out\" type=\"s\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_Cache =
"<interface name=\"org.a11y.atspi.Cache\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <method name=\"GetItems\">"
"      <arg direction=\"out\" name=\"nodes\" type=\"a((so)(so)(so)iiassusau)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiAccessibleCacheArray\" />"
"    </method>"
""
"    "
"    <signal name=\"AddAccessible\">"
"      <arg name=\"nodeAdded\" type=\"((so)(so)(so)iiassusau)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiAccessibleCacheItem\" />"
"    </signal>"
""
"    "
"    <signal name=\"RemoveAccessible\">"
"      <arg name=\"nodeRemoved\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiObjectReference\" />"
"    </signal>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_Collection =
"<interface name=\"org.a11y.atspi.Collection\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <method name=\"GetMatches\">"
"      <arg direction=\"in\" name=\"rule\" type=\"(aiia{ss}iaiiasib)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiMatchRule\" />"
"      <arg direction=\"in\" name=\"sortby\" type=\"u\" />"
"      <arg direction=\"in\" name=\"count\" type=\"i\" />"
"      <arg direction=\"in\" name=\"traverse\" type=\"b\" />"
"      <arg direction=\"out\" type=\"a(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiReferenceSet\" />"
"    </method>"
""
"    "
"    <method name=\"GetMatchesTo\">"
"      <arg direction=\"in\" name=\"currentObject\" type=\"o\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiObjectReference\" />"
"      <arg direction=\"in\" name=\"rule\" type=\"(aiia{ss}iaiiasib)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In1\" value=\"QSpiMatchRule\" />"
"      <arg direction=\"in\" name=\"sortby\" type=\"u\" />"
"      <arg direction=\"in\" name=\"tree\" type=\"u\" />"
"      <arg direction=\"in\" name=\"limit_scope\" type=\"b\" />"
"      <arg direction=\"in\" name=\"count\" type=\"i\" />"
"      <arg direction=\"in\" name=\"traverse\" type=\"b\" />"
"      <arg direction=\"out\" type=\"a(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiReferenceSet\" />"
"    </method>"
""
"    "
"    <method name=\"GetMatchesFrom\">"
"      <arg direction=\"in\" name=\"currentObject\" type=\"o\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiObjectReference\" />"
"      <arg direction=\"in\" name=\"rule\" type=\"(aiia{ss}iaiiasib)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In1\" value=\"QSpiMatchRule\" />"
"      <arg direction=\"in\" name=\"sortby\" type=\"u\" />"
"      <arg direction=\"in\" name=\"tree\" type=\"u\" />"
"      <arg direction=\"in\" name=\"count\" type=\"i\" />"
"      <arg direction=\"in\" name=\"traverse\" type=\"b\" />"
"      <arg direction=\"out\" type=\"a(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiReferenceSet\" />"
"    </method>"
""
"    "
"    <method name=\"GetActiveDescendant\">"
"      <arg direction=\"out\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiReferenceSet\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_Component =
"<interface name=\"org.a11y.atspi.Component\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <method name=\"Contains\">"
"      <arg direction=\"in\" name=\"x\" type=\"i\" />"
"      <arg direction=\"in\" name=\"y\" type=\"i\" />"
"      <arg direction=\"in\" name=\"coord_type\" type=\"u\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"GetAccessibleAtPoint\">"
"      <arg direction=\"in\" name=\"x\" type=\"i\" />"
"      <arg direction=\"in\" name=\"y\" type=\"i\" />"
"      <arg direction=\"in\" name=\"coord_type\" type=\"u\" />"
"      <arg direction=\"out\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiObjectReference\" />"
"    </method>"
""
"    "
"    <method name=\"GetExtents\">"
"      <arg direction=\"in\" name=\"coord_type\" type=\"u\" />"
"      <arg direction=\"out\" type=\"(iiii)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiRect\" />"
"    </method>"
""
"    "
"    <method name=\"GetPosition\">"
"      <arg direction=\"in\" name=\"coord_type\" type=\"u\" />"
"      <arg direction=\"out\" name=\"x\" type=\"i\" />"
"      <arg direction=\"out\" name=\"y\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetSize\">"
"      <arg direction=\"out\" name=\"width\" type=\"i\" />"
"      <arg direction=\"out\" name=\"height\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetLayer\">"
"      <arg direction=\"out\" type=\"u\" />"
"    </method>"
""
"    "
"    <method name=\"GetMDIZOrder\">"
"      <arg direction=\"out\" type=\"n\" />"
"    </method>"
""
"    "
"    <method name=\"GrabFocus\">"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"GetAlpha\">"
"      <arg direction=\"out\" type=\"d\" />"
"    </method>"
""
"    "
"    <method name=\"SetExtents\">"
"      <arg direction=\"in\" name=\"x\" type=\"i\" />"
"      <arg direction=\"in\" name=\"y\" type=\"i\" />"
"      <arg direction=\"in\" name=\"width\" type=\"i\" />"
"      <arg direction=\"in\" name=\"height\" type=\"i\" />"
"      <arg direction=\"in\" name=\"coord_type\" type=\"u\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"SetPosition\">"
"      <arg direction=\"in\" name=\"x\" type=\"i\" />"
"      <arg direction=\"in\" name=\"y\" type=\"i\" />"
"      <arg direction=\"in\" name=\"coord_type\" type=\"u\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"SetSize\">"
"      <arg direction=\"in\" name=\"width\" type=\"i\" />"
"      <arg direction=\"in\" name=\"height\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"ScrollTo\">"
"      <arg direction=\"in\" name=\"type\" type=\"u\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"ScrollToPoint\">"
"      <arg direction=\"in\" name=\"coord_type\" type=\"u\" />"
"      <arg direction=\"in\" name=\"x\" type=\"i\" />"
"      <arg direction=\"in\" name=\"y\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_DeviceEventController =
"<interface name=\"org.a11y.atspi.DeviceEventController\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <method name=\"RegisterKeystrokeListener\">"
"      <arg direction=\"in\" name=\"listener\" type=\"o\" />"
"      <arg direction=\"in\" name=\"keys\" type=\"a(iisi)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In1\" value=\"QSpiKeyTypeArray\" />"
"      <arg direction=\"in\" name=\"mask\" type=\"u\" />"
"      <arg direction=\"in\" name=\"types\" type=\"u\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In3\" value=\"QSpiEventTypeArray\" />"
"      <arg direction=\"in\" name=\"mode\" type=\"(bbb)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In4\" value=\"QSpiEventMode\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    <method name=\"DeregisterKeystrokeListener\">"
"      <arg direction=\"in\" name=\"listener\" type=\"o\" />"
"      <arg direction=\"in\" name=\"keys\" type=\"a(iisi)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In1\" value=\"QSpiKeyTypeArray\" />"
"      <arg direction=\"in\" name=\"mask\" type=\"u\" />"
"      <arg direction=\"in\" name=\"type\" type=\"u\" />"
"    </method>"
""
"    <method name=\"GetKeystrokeListeners\">"
"      <arg direction=\"out\" type=\"a(souua(iisi)u(bbb))\" />"
"    </method>"
""
"    <method name=\"GenerateKeyboardEvent\">"
"      <arg direction=\"in\" name=\"keycode\" type=\"i\" />"
"      <arg direction=\"in\" name=\"keystring\" type=\"s\" />"
"      <arg direction=\"in\" name=\"type\" type=\"u\" />"
"    </method>"
""
"    <method name=\"GenerateMouseEvent\">"
"      <arg direction=\"in\" name=\"x\" type=\"i\" />"
"      <arg direction=\"in\" name=\"y\" type=\"i\" />"
"      <arg direction=\"in\" name=\"eventName\" type=\"s\" />"
"    </method>"
""
"    <method name=\"NotifyListenersSync\">"
"      <arg direction=\"in\" name=\"event\" type=\"(uiuuisb)\" />"
"      <arg direction=\"out\" type=\"b\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiDeviceEvent\" />"
"    </method>"
""
"    <method name=\"NotifyListenersAsync\">"
"      <arg direction=\"in\" name=\"event\" type=\"(uiuuisb)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiDeviceEvent\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_DeviceEventListener =
"<interface name=\"org.a11y.atspi.DeviceEventListener\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <method name=\"NotifyEvent\">"
"      <arg direction=\"in\" name=\"event\" type=\"(uiuuisb)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiDeviceEvent\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <signal name=\"KeystrokeListenerRegistered\">"
"      <arg name=\"listener\" type=\"(souua(iisi)u(bbb))\" />"
"    </signal>"
""
"    "
"    <signal name=\"KeystrokeListenerDeregistered\">"
"      <arg name=\"listener\" type=\"(souua(iisi)u(bbb))\" />"
"    </signal>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_Document =
"<interface name=\"org.a11y.atspi.Document\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <property name=\"CurrentPageNumber\" type=\"i\" access=\"read\" />"
""
"    "
"    <property name=\"PageCount\" type=\"i\" access=\"read\" />"
""
"    "
"    <method name=\"GetLocale\">"
"      <arg direction=\"out\" type=\"s\" />"
"    </method>"
""
"    "
"    <method name=\"GetAttributeValue\">"
"      <arg direction=\"in\" name=\"attributename\" type=\"s\" />"
"      <arg direction=\"out\" type=\"s\" />"
"    </method>"
""
"    "
"    <method name=\"GetAttributes\">"
"      <arg direction=\"out\" type=\"a{ss}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiAttributeSet\" />"
"    </method>"
""
"    "
"    <method name=\"GetTextSelections\">"
"      <arg direction=\"out\" name=\"selections\" type=\"a((so)i(so)ib)\" />"
"    </method>"
""
"    "
"    <method name=\"SetTextSelections\">"
"      <arg direction=\"in\" name=\"selections\" type=\"a((so)i(so)ib)\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_EditableText =
"<interface name=\"org.a11y.atspi.EditableText\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <method name=\"SetTextContents\">"
"      <arg direction=\"in\" name=\"newContents\" type=\"s\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"InsertText\">"
"      <arg direction=\"in\" name=\"position\" type=\"i\" />"
"      <arg direction=\"in\" name=\"text\" type=\"s\" />"
"      <arg direction=\"in\" name=\"length\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"CopyText\">"
"      <arg direction=\"in\" name=\"startPos\" type=\"i\" />"
"      <arg direction=\"in\" name=\"endPos\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"CutText\">"
"      <arg direction=\"in\" name=\"startPos\" type=\"i\" />"
"      <arg direction=\"in\" name=\"endPos\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"DeleteText\">"
"      <arg direction=\"in\" name=\"startPos\" type=\"i\" />"
"      <arg direction=\"in\" name=\"endPos\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"PasteText\">"
"      <arg direction=\"in\" name=\"position\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_Event_Object =
"<interface name=\"org.a11y.atspi.Event.Object\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    <signal name=\"PropertyChange\">"
"      <arg name=\"property\" type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg name=\"value\" type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"BoundsChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" name=\"bounds\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"LinkSelected\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"StateChanged\">"
"      <arg name=\"state\" type=\"s\" />"
"      <arg name=\"enabled\" type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"ChildrenChanged\">"
"      <arg name=\"operation\" type=\"s\" />"
"      <arg name=\"index_in_parent\" type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg name=\"child\" type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"VisibleDataChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"SelectionChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"ModelChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"ActiveDescendantChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg name=\"child\" type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Announcement\">"
"      <arg type=\"s\" />"
"      "
"      <arg type=\"i\" name=\"politeness\" />"
"      <arg type=\"i\" />"
"      "
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"AttributesChanged\">"
"      "
"      <arg name=\"name\" type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      "
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    "
""
"    <signal name=\"RowInserted\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"RowReordered\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"RowDeleted\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"ColumnInserted\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"ColumnReordered\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"ColumnDeleted\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    "
""
"    <signal name=\"TextBoundsChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"TextSelectionChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"TextChanged\">"
"      <arg name=\"detail\" type=\"s\" />"
"      <arg name=\"start_pos\" type=\"i\" />"
"      <arg name=\"length\" type=\"i\" />"
"      <arg name=\"text\" type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"TextAttributesChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"TextCaretMoved\">"
"      <arg type=\"s\" />"
"      <arg name=\"position\" type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
"  </interface>"
""
"  ";

const char *spi_org_a11y_atspi_Event_Window =
"<interface name=\"org.a11y.atspi.Event.Window\">"
"    <signal name=\"PropertyChange\">"
"      <arg name=\"property\" type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Minimize\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Maximize\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Restore\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Close\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Create\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Reparent\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"DesktopCreate\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"DesktopDestroy\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Destroy\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Activate\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Deactivate\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Raise\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Lower\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Move\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Resize\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Shade\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"uUshade\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Restyle\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
"  </interface>"
""
"  ";

const char *spi_org_a11y_atspi_Event_Mouse =
"<interface name=\"org.a11y.atspi.Event.Mouse\">"
""
"    <signal name=\"Abs\">"
"      <arg type=\"s\" />"
"      <arg name=\"x\" type=\"i\" />"
"      <arg name=\"y\" type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Rel\">"
"      <arg type=\"s\" />"
"      <arg name=\"x\" type=\"i\" />"
"      <arg name=\"y\" type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Button\">"
"      <arg name=\"detail\" type=\"s\" />"
"      <arg name=\"mouse_x\" type=\"i\" />"
"      <arg name=\"mouse_y\" type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
"  </interface>"
""
"  ";

const char *spi_org_a11y_atspi_Event_Keyboard =
"<interface name=\"org.a11y.atspi.Event.Keyboard\">"
""
"    <signal name=\"Modifiers\">"
"      <arg type=\"s\" />"
"      <arg name=\"previous_modifiers\" type=\"i\" />"
"      <arg name=\"current_modifiers\" type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
"  </interface>"
""
"  ";

const char *spi_org_a11y_atspi_Event_Terminal =
"<interface name=\"org.a11y.atspi.Event.Terminal\">"
""
"    <signal name=\"LineChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"ColumncountChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"LinecountChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"ApplicationChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"CharwidthChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
"  </interface>"
""
"  ";

const char *spi_org_a11y_atspi_Event_Document =
"<interface name=\"org.a11y.atspi.Event.Document\">"
""
"    <signal name=\"LoadComplete\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"Reload\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"LoadStopped\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"ContentChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"AttributesChanged\">"
"      "
"      <arg name=\"name\" type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      "
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
""
"    <signal name=\"PageChanged\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
"  </interface>"
""
"  ";

const char *spi_org_a11y_atspi_Event_Focus =
"<interface name=\"org.a11y.atspi.Event.Focus\">"
"    "
"    <signal name=\"Focus\">"
"      <arg type=\"s\" />"
"      <arg type=\"i\" />"
"      <arg type=\"i\" />"
"      <arg type=\"v\" />"
"      <arg name=\"properties\" type=\"a{sv}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiEvent\" />"
"    </signal>"
"  </interface>"
""
"";

const char *spi_org_a11y_atspi_Hyperlink =
"<interface name=\"org.a11y.atspi.Hyperlink\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <property name=\"NAnchors\" type=\"n\" access=\"read\" />"
""
"    "
"    <property name=\"StartIndex\" type=\"i\" access=\"read\" />"
""
"    "
"    <property name=\"EndIndex\" type=\"i\" access=\"read\" />"
""
"    "
"    <method name=\"GetObject\">"
"      <arg direction=\"in\" name=\"i\" type=\"i\" />"
"      <arg direction=\"out\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiObjectReference\" />"
"    </method>"
""
"    "
"    <method name=\"GetURI\">"
"      <arg direction=\"in\" name=\"i\" type=\"i\" />"
"      <arg direction=\"out\" type=\"s\" />"
"    </method>"
""
"    "
"    <method name=\"IsValid\">"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_Hypertext =
"<interface name=\"org.a11y.atspi.Hypertext\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <method name=\"GetNLinks\">"
"      <arg direction=\"out\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetLink\">"
"      <arg direction=\"in\" name=\"linkIndex\" type=\"i\" />"
"      <arg direction=\"out\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiObjectReference\" />"
"    </method>"
""
"    "
"    <method name=\"GetLinkIndex\">"
"      <arg direction=\"in\" name=\"characterIndex\" type=\"i\" />"
"      <arg direction=\"out\" type=\"i\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_Image =
"<interface name=\"org.a11y.atspi.Image\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <property name=\"ImageDescription\" type=\"s\" access=\"read\" />"
""
"    "
"    <property name=\"ImageLocale\" type=\"s\" access=\"read\" />"
""
"    "
"    <method name=\"GetImageExtents\">"
"      <arg direction=\"in\" name=\"coordType\" type=\"u\" />"
"      <arg direction=\"out\" type=\"(iiii)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiRect\" />"
"    </method>"
""
"    "
"    <method name=\"GetImagePosition\">"
"      <arg direction=\"in\" name=\"coordType\" type=\"u\" />"
"      <arg direction=\"out\" name=\"x\" type=\"i\" />"
"      <arg direction=\"out\" name=\"y\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetImageSize\">"
"      <arg direction=\"out\" name=\"width\" type=\"i\" />"
"      <arg direction=\"out\" name=\"height\" type=\"i\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_Registry =
"<interface name=\"org.a11y.atspi.Registry\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <method name=\"RegisterEvent\">"
"      <arg direction=\"in\" name=\"event\" type=\"s\" />"
"      <arg direction=\"in\" name=\"properties\" type=\"as\" />"
"      <arg direction=\"in\" name=\"app_bus_name\" type=\"s\" />"
"    </method>"
""
"    "
"    <method name=\"DeregisterEvent\">"
"      <arg direction=\"in\" name=\"event\" type=\"s\" />"
"      <arg direction=\"in\" name=\"app_bus_name\" type=\"s\" />"
"    </method>"
""
"    "
"    <method name=\"GetRegisteredEvents\">"
"      <arg direction=\"out\" name=\"events\" type=\"a(ss)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiEventListenerArray\" />"
"    </method>"
""
"    "
"    <signal name=\"EventListenerRegistered\">"
"      <arg name=\"bus\" type=\"s\" />"
"      <arg name=\"event\" type=\"s\" />"
"      <arg name=\"properties\" type=\"as\" />"
"    </signal>"
""
"    "
"    <signal name=\"EventListenerDeregistered\">"
"      <arg name=\"bus\" type=\"s\" />"
"      <arg name=\"event\" type=\"s\" />"
"    </signal>"
"  </interface>"
"";

const char *spi_org_a11y_atspi_Selection =
"<interface name=\"org.a11y.atspi.Selection\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <property name=\"NSelectedChildren\" type=\"i\" access=\"read\" />"
""
"    "
"    <method name=\"GetSelectedChild\">"
"      <arg direction=\"in\" name=\"selectedChildIndex\" type=\"i\" />"
"      <arg direction=\"out\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiObjectReference\" />"
"    </method>"
""
"    "
"    <method name=\"SelectChild\">"
"      <arg direction=\"in\" name=\"childIndex\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"DeselectSelectedChild\">"
"      <arg direction=\"in\" name=\"selectedChildIndex\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
"    "
"    <method name=\"IsChildSelected\">"
"      <arg direction=\"in\" name=\"childIndex\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"SelectAll\">"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"ClearSelection\">"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"DeselectChild\">"
"      <arg direction=\"in\" name=\"childIndex\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_Socket =
"<interface name=\"org.a11y.atspi.Socket\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <method name=\"Embed\">"
"      <arg direction=\"in\" name=\"plug\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiObjectReference\" />"
"      <arg direction=\"out\" name=\"socket\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiObjectReference\" />"
"    </method>"
""
"    "
"    <method name=\"Embedded\">"
"      <arg direction=\"in\" name=\"path\" type=\"s\" />"
"    </method>"
""
"    "
"    <method name=\"Unembed\">"
"      <arg direction=\"in\" name=\"plug\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiObjectReference\" />"
"    </method>"
""
"    "
"    <signal name=\"Available\">"
"      <arg name=\"socket\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.In0\" value=\"QSpiObjectReference\" />"
"    </signal>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_Table =
"<interface name=\"org.a11y.atspi.Table\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <property name=\"NRows\" type=\"i\" access=\"read\" />"
""
"    "
"    <property name=\"NColumns\" type=\"i\" access=\"read\" />"
""
"    "
"    <property name=\"Caption\" type=\"(so)\" access=\"read\">"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName\" value=\"QSpiObjectReference\" />"
"    </property>"
""
"    "
"    <property name=\"Summary\" type=\"(so)\" access=\"read\">"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName\" value=\"QSpiObjectReference\" />"
"    </property>"
""
"    "
"    <property name=\"NSelectedRows\" type=\"i\" access=\"read\" />"
""
"    "
"    <property name=\"NSelectedColumns\" type=\"i\" access=\"read\" />"
""
"    "
"    <method name=\"GetAccessibleAt\">"
"      <arg direction=\"in\" name=\"row\" type=\"i\" />"
"      <arg direction=\"in\" name=\"column\" type=\"i\" />"
"      <arg direction=\"out\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiObjectReference\" />"
"    </method>"
""
"    "
"    <method name=\"GetIndexAt\">"
"      <arg direction=\"in\" name=\"row\" type=\"i\" />"
"      <arg direction=\"in\" name=\"column\" type=\"i\" />"
"      <arg direction=\"out\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetRowAtIndex\">"
"      <arg direction=\"in\" name=\"index\" type=\"i\" />"
"      <arg direction=\"out\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetColumnAtIndex\">"
"      <arg direction=\"in\" name=\"index\" type=\"i\" />"
"      <arg direction=\"out\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetRowDescription\">"
"      <arg direction=\"in\" name=\"row\" type=\"i\" />"
"      <arg direction=\"out\" type=\"s\" />"
"    </method>"
""
"    "
"    <method name=\"GetColumnDescription\">"
"      <arg direction=\"in\" name=\"column\" type=\"i\" />"
"      <arg direction=\"out\" type=\"s\" />"
"    </method>"
""
"    "
"    <method name=\"GetRowExtentAt\">"
"      <arg direction=\"in\" name=\"row\" type=\"i\" />"
"      <arg direction=\"in\" name=\"column\" type=\"i\" />"
"      <arg direction=\"out\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetColumnExtentAt\">"
"      <arg direction=\"in\" name=\"row\" type=\"i\" />"
"      <arg direction=\"in\" name=\"column\" type=\"i\" />"
"      <arg direction=\"out\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetRowHeader\">"
"      <arg direction=\"in\" name=\"row\" type=\"i\" />"
"      <arg direction=\"out\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiObjectReference\" />"
"    </method>"
""
"    "
"    <method name=\"GetColumnHeader\">"
"      <arg direction=\"in\" name=\"column\" type=\"i\" />"
"      <arg direction=\"out\" type=\"(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiObjectReference\" />"
"    </method>"
""
"    "
"    <method name=\"GetSelectedRows\">"
"      <arg direction=\"out\" type=\"ai\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiIntList\" />"
"    </method>"
""
"    "
"    <method name=\"GetSelectedColumns\">"
"      <arg direction=\"out\" type=\"ai\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiIntList\" />"
"    </method>"
""
"    "
"    <method name=\"IsRowSelected\">"
"      <arg direction=\"in\" name=\"row\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"IsColumnSelected\">"
"      <arg direction=\"in\" name=\"column\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"IsSelected\">"
"      <arg direction=\"in\" name=\"row\" type=\"i\" />"
"      <arg direction=\"in\" name=\"column\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"AddRowSelection\">"
"      <arg direction=\"in\" name=\"row\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"AddColumnSelection\">"
"      <arg direction=\"in\" name=\"column\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"RemoveRowSelection\">"
"      <arg direction=\"in\" name=\"row\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"RemoveColumnSelection\">"
"      <arg direction=\"in\" name=\"column\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"GetRowColumnExtentsAtIndex\">"
"      <arg direction=\"in\" name=\"index\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"      <arg direction=\"out\" name=\"row\" type=\"i\" />"
"      <arg direction=\"out\" name=\"col\" type=\"i\" />"
"      <arg direction=\"out\" name=\"row_extents\" type=\"i\" />"
"      <arg direction=\"out\" name=\"col_extents\" type=\"i\" />"
"      <arg direction=\"out\" name=\"is_selected\" type=\"b\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_TableCell =
"<interface name=\"org.a11y.atspi.TableCell\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <property access=\"read\" name=\"ColumnSpan\" type=\"i\" />"
""
"    "
"    <property access=\"read\" name=\"Position\" type=\"(ii)\">"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName\" value=\"QPoint\" />"
"    </property>"
""
"    "
"    <property access=\"read\" name=\"RowSpan\" type=\"i\" />"
""
"    "
"    <property access=\"read\" name=\"Table\" type=\"(so)\">"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName\" value=\"QSpiObjectReference\" />"
"    </property>"
""
"    "
"    <method name=\"GetRowColumnSpan\">"
"      <arg direction=\"out\" type=\"b\" />"
"      <arg direction=\"out\" name=\"row\" type=\"i\" />"
"      <arg direction=\"out\" name=\"col\" type=\"i\" />"
"      <arg direction=\"out\" name=\"row_extents\" type=\"i\" />"
"      <arg direction=\"out\" name=\"col_extents\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetColumnHeaderCells\">"
"      <arg direction=\"out\" type=\"a(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiObjectReferenceArray\" />"
"    </method>"
""
"    "
"    <method name=\"GetRowHeaderCells\">"
"      <arg direction=\"out\" type=\"a(so)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiObjectReferenceArray\" />"
"    </method>"
"  </interface>"
"";

const char *spi_org_a11y_atspi_Text =
"<interface name=\"org.a11y.atspi.Text\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <property name=\"CharacterCount\" type=\"i\" access=\"read\" />"
""
"    "
"    <property name=\"CaretOffset\" type=\"i\" access=\"read\" />"
""
"    "
"    <method name=\"GetStringAtOffset\">"
"      <arg direction=\"in\" name=\"offset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"granularity\" type=\"u\" />"
"      <arg direction=\"out\" type=\"s\" />"
"      <arg direction=\"out\" name=\"startOffset\" type=\"i\" />"
"      <arg direction=\"out\" name=\"endOffset\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetText\">"
"      <arg direction=\"in\" name=\"startOffset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"endOffset\" type=\"i\" />"
"      <arg direction=\"out\" type=\"s\" />"
"    </method>"
""
"    "
"    <method name=\"SetCaretOffset\">"
"      <arg direction=\"in\" name=\"offset\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"GetTextBeforeOffset\">"
"      <arg direction=\"in\" name=\"offset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"type\" type=\"u\" />"
"      <arg direction=\"out\" type=\"s\" />"
"      <arg direction=\"out\" name=\"startOffset\" type=\"i\" />"
"      <arg direction=\"out\" name=\"endOffset\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetTextAtOffset\">"
"      <arg direction=\"in\" name=\"offset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"type\" type=\"u\" />"
"      <arg direction=\"out\" type=\"s\" />"
"      <arg direction=\"out\" name=\"startOffset\" type=\"i\" />"
"      <arg direction=\"out\" name=\"endOffset\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetTextAfterOffset\">"
"      <arg direction=\"in\" name=\"offset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"type\" type=\"u\" />"
"      <arg direction=\"out\" type=\"s\" />"
"      <arg direction=\"out\" name=\"startOffset\" type=\"i\" />"
"      <arg direction=\"out\" name=\"endOffset\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetCharacterAtOffset\">"
"      <arg direction=\"in\" name=\"offset\" type=\"i\" />"
"      <arg direction=\"out\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetAttributeValue\">"
"      <arg direction=\"in\" name=\"offset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"attributeName\" type=\"s\" />"
"      <arg direction=\"out\" type=\"s\" />"
"    </method>"
""
"    "
"    <method name=\"GetAttributes\">"
"      <arg direction=\"in\" name=\"offset\" type=\"i\" />"
"      <arg direction=\"out\" type=\"a{ss}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiAttributeSet\" />"
"      <arg direction=\"out\" name=\"startOffset\" type=\"i\" />"
"      <arg direction=\"out\" name=\"endOffset\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetDefaultAttributes\">"
"      <arg direction=\"out\" type=\"a{ss}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiAttributeSet\" />"
"    </method>"
""
"    "
"    <method name=\"GetCharacterExtents\">"
"      <arg direction=\"in\" name=\"offset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"coordType\" type=\"u\" />"
"      <arg direction=\"out\" name=\"x\" type=\"i\" />"
"      <arg direction=\"out\" name=\"y\" type=\"i\" />"
"      <arg direction=\"out\" name=\"width\" type=\"i\" />"
"      <arg direction=\"out\" name=\"height\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetOffsetAtPoint\">"
"      <arg direction=\"in\" name=\"x\" type=\"i\" />"
"      <arg direction=\"in\" name=\"y\" type=\"i\" />"
"      <arg direction=\"in\" name=\"coordType\" type=\"u\" />"
"      <arg direction=\"out\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetNSelections\">"
"      <arg direction=\"out\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetSelection\">"
"      <arg direction=\"in\" name=\"selectionNum\" type=\"i\" />"
"      <arg direction=\"out\" name=\"startOffset\" type=\"i\" />"
"      <arg direction=\"out\" name=\"endOffset\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"AddSelection\">"
"      <arg direction=\"in\" name=\"startOffset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"endOffset\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"RemoveSelection\">"
"      <arg direction=\"in\" name=\"selectionNum\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"SetSelection\">"
"      <arg direction=\"in\" name=\"selectionNum\" type=\"i\" />"
"      <arg direction=\"in\" name=\"startOffset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"endOffset\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"GetRangeExtents\">"
"      <arg direction=\"in\" name=\"startOffset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"endOffset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"coordType\" type=\"u\" />"
"      <arg direction=\"out\" name=\"x\" type=\"i\" />"
"      <arg direction=\"out\" name=\"y\" type=\"i\" />"
"      <arg direction=\"out\" name=\"width\" type=\"i\" />"
"      <arg direction=\"out\" name=\"height\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetBoundedRanges\">"
"      <arg direction=\"in\" name=\"x\" type=\"i\" />"
"      <arg direction=\"in\" name=\"y\" type=\"i\" />"
"      <arg direction=\"in\" name=\"width\" type=\"i\" />"
"      <arg direction=\"in\" name=\"height\" type=\"i\" />"
"      <arg direction=\"in\" name=\"coordType\" type=\"u\" />"
"      <arg direction=\"in\" name=\"xClipType\" type=\"u\" />"
"      <arg direction=\"in\" name=\"yClipType\" type=\"u\" />"
"      <arg direction=\"out\" type=\"a(iisv)\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiRangeList\" />"
"    </method>"
""
"    "
"    <method name=\"GetAttributeRun\">"
"      <arg direction=\"in\" name=\"offset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"includeDefaults\" type=\"b\" />"
"      <arg direction=\"out\" type=\"a{ss}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiAttributeSet\" />"
"      <arg direction=\"out\" name=\"startOffset\" type=\"i\" />"
"      <arg direction=\"out\" name=\"endOffset\" type=\"i\" />"
"    </method>"
""
"    "
"    <method name=\"GetDefaultAttributeSet\">"
"      <arg direction=\"out\" type=\"a{ss}\" />"
"      <annotation name=\"org.qtproject.QtDBus.QtTypeName.Out0\" value=\"QSpiAttributeSet\" />"
"    </method>"
""
"    "
"    <method name=\"ScrollSubstringTo\">"
"      <arg direction=\"in\" name=\"startOffset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"endOffset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"type\" type=\"u\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"    "
"    <method name=\"ScrollSubstringToPoint\">"
"      <arg direction=\"in\" name=\"startOffset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"endOffset\" type=\"i\" />"
"      <arg direction=\"in\" name=\"coordType\" type=\"u\" />"
"      <arg direction=\"in\" name=\"x\" type=\"i\" />"
"      <arg direction=\"in\" name=\"y\" type=\"i\" />"
"      <arg direction=\"out\" type=\"b\" />"
"    </method>"
""
"  </interface>"
"";

const char *spi_org_a11y_atspi_Value =
"<interface name=\"org.a11y.atspi.Value\">"
""
"    "
"    <property name=\"version\" type=\"u\" access=\"read\" />"
""
"    "
"    <property name=\"MinimumValue\" type=\"d\" access=\"read\" />"
""
"    "
"    <property name=\"MaximumValue\" type=\"d\" access=\"read\" />"
""
"    "
"    <property name=\"MinimumIncrement\" type=\"d\" access=\"read\" />"
""
"    "
"    <property name=\"CurrentValue\" type=\"d\" access=\"readwrite\" />"
""
"    "
"    <property name=\"Text\" type=\"s\" access=\"read\" />"
""
"  </interface>"
"";

